// ignore_for_file: avoid_print

library tim_ui_kit_calling_plugin;

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:tencent_im_base/tencent_im_base.dart';
import 'package:tim_ui_kit_calling_plugin/enum/tim_uikit_trtc_call_conv_type.dart';
import 'package:tim_ui_kit_calling_plugin/enum/tim_uikit_trtc_calling_scence.dart';
import 'package:tim_ui_kit_calling_plugin/model/TIMUIKitCallingListener.dart';
import 'package:tim_ui_kit_calling_plugin/model/TRTCCalling.dart';
import 'package:tim_ui_kit_calling_plugin/model/TRTCCallingDef.dart';
import 'package:tim_ui_kit_calling_plugin/view/tim_uikit_trtc.dart';

export 'package:tencent_trtc_cloud/trtc_cloud_def.dart';
export 'package:tencent_im_base/class/calling_message.dart';
export 'package:tencent_trtc_cloud/trtc_cloud_video_view.dart';

class TUICalling {
  static bool _enableCustomViewRoute = false;
  static bool _enableFloatingWindow = false;
  static GlobalKey<NavigatorState> navigatorKey = GlobalKey<NavigatorState>();
  static late V2TimUserFullInfo? _loginUserInfo;
  static late TRTCCalling _tRTCCallingService;
  static bool _isInit = false;
  static late TUICallingListener _onRtcListener;
  static bool _isCanAccept = true;
  OverlayEntry? _entry;

   Future<void> init({
    required int sdkAppID,
    required String userID,
    required String userSig,
    BuildContext? context
  }) async {
    _tRTCCallingService = await TRTCCalling.sharedInstance();
    try {
      ActionCallback res =
          await _tRTCCallingService.login(sdkAppID, userID, userSig);
      print("calling-plugin login response $res");
    } catch (err) {
      print("calling-plugin login fail $err");
    }
    _loginUserInfo = (await _getUserInfo([userID]))!.first;
    _registerListener();
    _isInit = true;
    return;
  }

  enableCustomViewRoute({
    bool isEnable = true,
  }) {
    _enableCustomViewRoute = isEnable;
  }

  enableFloatingWindow({
    bool isEnable = true,
  }) {
    _enableFloatingWindow = isEnable;
  }

  Future<void> _registerListener() async {
    _onRtcListener = TUICallingListener(onInvited: (params) async {
      print('onInvited');
      _isCanAccept = true;
      if (_enableCustomViewRoute) {
        return;
      }
      final inviterUserID = params.sponsor;

      // 被邀请方用户ID数组，不包含自己
      final inviteUserIDs = params.userIDList;

      // 通话场景 语音通话/视频通话
      final callScenes = params.callType;

      /// 是否群通话
      final isFromGroup = params.isFromGroup;

      final groupID = params.groupID;

      /// 邀请方用户信息
      final inviterUserInfo = await _getUserInfo([inviterUserID]);

      if (_loginUserInfo == null) {
        return;
      }

      /// 被邀请方用户信息
      List<V2TimUserFullInfo> inviteUsersInfoList = (inviteUserIDs.isNotEmpty
          ? [_loginUserInfo!, ...?await _getUserInfo(inviteUserIDs)]
          : [_loginUserInfo!]);

      /// 是否主动邀请发起通话
      const isInitiationCalling = false;

      _buildCallingOverlay(
          isFromGroup ? ConvType.group : ConvType.c2c,
          inviteUsersInfoList,
          inviterUserInfo!.first,
          callScenes == "audioCall" ? CallingScenes.Audio : CallingScenes.Video,
          isInitiationCalling,
          groupID);
    }, onCallingCancel: () {
      print('onCallingCancel');
      if (_entry != null) {
        _entry?.remove();
        _entry = null;
      } else {
        _isCanAccept = false;
        _entry?.remove();
        _entry = null;
      }
    });
    await _tRTCCallingService.registerListener(_onRtcListener);
    return;
  }

  destroy() {
    if (!_isInit) {
      throw Exception("not init");
    }
    _tRTCCallingService.destroy();
  }

  Future<List<V2TimUserFullInfo>?> _getUserInfo(List<String> userIDList) async {
    final res = await TencentImSDKPlugin.v2TIMManager
        .getUsersInfo(userIDList: userIDList);
    return res.data;
  }

  Future<TRTCCalling> sharedInstance() {
    return TRTCCalling.sharedInstance();
  }

  Future<void> call(
    String userId,
    CallingScenes type, [
    OfflinePushInfo? offlinePushInfo,
  ]) async {
    _isCanAccept = true;
    if (_enableCustomViewRoute) {
      return;
    }

    /// 被邀请方用户信息
    final inviteUsersInfoList = await _getUserInfo([userId]);

    /// 邀请方
    final inviterUserInfo = _loginUserInfo;

    if (inviterUserInfo == null) {
      return;
    }

    /// 1:语音通话 2:视频通话
    final callScenes = type;

    /// 是否是主动发起通话
    const isInitiationCalling = true;

    _buildCallingOverlay(ConvType.c2c, inviteUsersInfoList, inviterUserInfo,
        callScenes, isInitiationCalling, null, offlinePushInfo);

    return;
  }

  Future<void> groupCall(
      List<String> userIdList, CallingScenes type, String? groupId) async {
    _isCanAccept = true;
    if (_enableCustomViewRoute) {
      return;
    }

    /// 被邀请方用户信息
    final inviteUsersInfoList = await _getUserInfo(userIdList);

    /// 邀请方
    final inviterUserInfo = _loginUserInfo;

    if (inviteUsersInfoList == null || inviterUserInfo == null) {
      return;
    }

    /// 语音通话:CallingScenes.Video
    /// 视频通话:CallingScenes.Audio
    final callScenes = type;

    /// 是否是主动发起通话
    const isInitiationCalling = true;

    if (inviteUsersInfoList.length + 1 > 9) {
      return;
    }

    _buildCallingOverlay(ConvType.group, inviteUsersInfoList, inviterUserInfo,
        callScenes, isInitiationCalling, groupId);

    return;
  }

  _buildCallingOverlay(convType, inviteUsersInfoList, inviterUserInfo,
      callScenes, isInitiationCalling,
      [String? groupId, OfflinePushInfo? offlinePushInfo]) async {
      SystemChannels.textInput.invokeMethod('TextInput.hide');
      final entry = OverlayEntry(builder: (context) {
        return TIMUIKitTRTCContainer(
            convType: convType,
            inviteUsersInfoList: inviteUsersInfoList!,
            inviterUserInfo: inviterUserInfo,
            callScenes: callScenes,
            isInitiationCalling: isInitiationCalling,
            groupID: groupId,
            enableFloatingWindow: _enableFloatingWindow,
            offlinePushInfo: offlinePushInfo,
            close: () {
              _entry?.remove();
              print("_entry_first ${_entry.toString()}");
              _entry = null;
              print("_entry_second ${_entry.toString()}");
            });
      });
      _entry = entry;
      if (_isCanAccept) {
        navigatorKey.currentState?.overlay?.insert(entry);
      }
  }

  setCallingListener(TUICallingListener listener) {
    _tRTCCallingService.registerListener(listener);
  }

  removeCallingListener(TUICallingListener listener) {
    _tRTCCallingService.unRegisterListener(listener);
  }
}
